using System;
using System.Windows.Forms;
using System.Drawing;
using System.Text;
using System.Collections.Generic;


namespace CodeJotter {
    public partial class CodeJotter : UserControl {

        private int tabWidth = 8;
        /// <summary>
        /// Width of tabs
        /// </summary>
        public int TabWidth {
            get { return tabWidth; }
            set {
                if (value < 0) throw new Exception("Tab width should be greater than 0.");
                tabWidth = value;
            }
        }

        private bool drawWhitespaceGlyphs = false;
        /// <summary>
        /// Display whitespace characters with special symbols
        /// </summary>
        public bool DrawWhitespaceGlyphs {
            get { return drawWhitespaceGlyphs; }
            set {
                if (drawWhitespaceGlyphs == value) return;
                drawWhitespaceGlyphs = value;
                Invalidate();
            }
        }

        private Brush lineNumberBrush = new SolidBrush(Color.FromArgb(43, 145, 175));
        /// <summary>
        /// Colour of the line numbers
        /// </summary>
        public Color LineNumbersColour {
            get { return ((SolidBrush)lineNumberBrush).Color; }
            set { lineNumberBrush = new SolidBrush(value); }
        }

        private Brush highlightBrush = new SolidBrush(Color.FromKnownColor(KnownColor.Highlight));
        public Color HighlightBackgroundColour {
            get { return ((SolidBrush)highlightBrush).Color; }
            set { highlightBrush = new SolidBrush(value); }
        }

        private Brush highlightTextBrush = new SolidBrush(Color.FromKnownColor(KnownColor.HighlightText));
        public Color HighlightTextColour {
            get { return ((SolidBrush)highlightTextBrush).Color; }
            set { highlightTextBrush = new SolidBrush(value); }
        }

        private Brush inactiveHighlightBrush = new SolidBrush(Color.FromKnownColor(KnownColor.InactiveCaption));
        public Color InactiveHighlightBackgroundColour {
            get { return ((SolidBrush)inactiveHighlightBrush).Color; }
            set { inactiveHighlightBrush = new SolidBrush(value); }
        }

        private Brush inactiveHighlightTextBrush = new SolidBrush(Color.FromKnownColor(KnownColor.InactiveCaptionText));
        public Color InactiveHighlightTextColour {  
            get { return ((SolidBrush)inactiveHighlightTextBrush).Color; }
            set { inactiveHighlightTextBrush = new SolidBrush(value); }
        }



        protected override void OnFontChanged(EventArgs e) {
            Application.DoEvents();
            Graphics g = this.CreateGraphics();
            GraphicsUnit gu = g.PageUnit;
            gu = GraphicsUnit.Pixel;
            g.PageUnit = gu;
            fontWidth = g.MeasureString("M", this.Font, 100, StringFormat.GenericTypographic).Width;
            fontHeight = Math.Max(13, this.Font.Height);
            base.OnFontChanged(e);
        }

        public string SelectedText {

            get {
                StringBuilder selText = new StringBuilder(lines.Count * 32);
                UpdatePointers();

                LinkedListNode<Line> upperLine = currentLine;
                LinkedListNode<char> upperChar = currentChar;
                LinkedListNode<Line> lowerLine = selectedLine;
                LinkedListNode<char> lowerChar = selectedChar;

                if (SelectionCursorPos.Y < CursorPos.Y || (SelectionCursorPos.Y == CursorPos.Y && SelectionCursorPos.X < CursorPos.X)) {
                    LinkedListNode<Line> s1 = upperLine; upperLine = lowerLine; lowerLine = s1;
                    LinkedListNode<char> s2 = upperChar; upperChar = lowerChar; lowerChar = s2;
                }

                while (!(upperChar.Next == null || upperChar == lowerChar)) {
                    selText.Append(upperChar.Value);
                    upperChar = upperChar.Next;
                }
                bool First = true;
                while (!(upperLine.Next == null || upperLine == lowerLine)) {
                    if (First) {
                        First = false;
                    } else {
                        selText.Append("\r\n");
                        foreach (char c in upperLine.Value.Characters) {
                            if (c != '\r') selText.Append(c);
                        }
                    }
                    upperLine = upperLine.Next;
                }

                if (SelectionCursorPos.Y != CursorPos.Y) {
                    StringBuilder LastLine = new StringBuilder(lowerLine.Value.Characters.Count);
                    lowerChar = lowerChar.Previous;
                    while (lowerChar != null) {
                        LastLine.Insert(0, lowerChar.Value);
                        lowerChar = lowerChar.Previous;
                    }
                    selText.Append("\r\n");
                    selText.Append(LastLine);
                }

                return selText.ToString();
            }
        }

        private bool HasSomethingSelected {
            get {
                return CursorPos != SelectionCursorPos;
            }
        }

        private SolidBrush[] markColours = new SolidBrush[] { new SolidBrush(Color.FromKnownColor(KnownColor.Window)), new SolidBrush(Color.FromArgb(255, 238, 98)), new SolidBrush(Color.FromArgb(108, 226, 108)) };
        
        public Color MarkUneditedColour {
            get { return markColours[(int)Line.SaveState.Unedited].Color; }
            set { markColours[(int)Line.SaveState.Unedited] = new SolidBrush(value); InvalidateAll(); }
        }

        public Color MarkDirtyColour {
            get { return markColours[(int)Line.SaveState.Dirty].Color; }
            set { markColours[(int)Line.SaveState.Dirty] = new SolidBrush(value); InvalidateAll(); }
        }

        public Color MarkSavedColour {
            get { return markColours[(int)Line.SaveState.Saved].Color; }
            set { markColours[(int)Line.SaveState.Saved] = new SolidBrush(value); InvalidateAll(); }
        }

        private SolidBrush currentLineHighlight = new SolidBrush(Color.FromKnownColor(KnownColor.Info));
        /// <summary>
        /// Colour of the current working line's highlight bar
        /// </summary>
        public Color LineHighlight {
            get { return currentLineHighlight.Color; }
            set { currentLineHighlight = new SolidBrush(value); }
        }

        private SolidBrush breakpointHighlight = new SolidBrush(Color.Maroon);
        /// <summary>
        /// The colour highlight of a line marked as a breakpoint
        /// </summary>
        public Color BreakpointHighlight {
            get { return breakpointHighlight.Color; }
            set { breakpointHighlight = new SolidBrush(value); }
        }

        private SolidBrush breakpointHighlightText = new SolidBrush(Color.White);
        /// <summary>
        /// The colour of text in a  line marked as a breakpoint
        /// </summary>
        public Color BreakpointHighlightText {
            get { return breakpointHighlightText.Color; }
            set { breakpointHighlightText = new SolidBrush(value); }
        }

    }
}
